@@ -11,8 +11,8 @@
 %% -compile(export_all).
 
 
--include_lib("yaws/include/yaws.hrl").
--include_lib("yaws/include/yaws_api.hrl").
+-include("../include/yaws.hrl").
+-include("../include/yaws_api.hrl").
 -include("yaws_debug.hrl").
 
 
@@ -56,7 +56,7 @@
 	ehtml_expander_test/0]).
 
 -export([parse_set_cookie/1, format_set_cookie/1, 
-	 postvar/2, queryvar/2]).
+	 postvar/2, queryvar/2, getvar/2]).
 
 -export([binding/1]).
 
@@ -726,19 +726,6 @@ rest_dir (N, Path, [  _H | T ] ) -> rest_dir (N    ,        Path  , T).
 url_decode_q_split(Path) ->
     url_decode_q_split(Path, []).
 
-% url_decode_q_split([$%, $C, $2, $%, Hi, Lo | Tail], Ack) ->
-%     Hex = yaws:hex_to_integer([Hi, Lo]),
-%     url_decode_q_split(Tail, [Hex|Ack]);
-% url_decode_q_split([$%, $C, $3, $%, Hi, Lo | Tail], Ack) when Hi > $9 ->
-%     Hex = yaws:hex_to_integer([Hi+4, Lo]),
-%     url_decode_q_split(Tail, [Hex|Ack]);
-% url_decode_q_split([$%, $C, $3, $%, Hi, Lo | Tail], Ack) when Hi < $A ->
-%     Hex = yaws:hex_to_integer([Hi+4+7, Lo]),
-%    url_decode_q_split(Tail, [Hex|Ack]);
-
-
-%% Removed the above hackery, (klacke)
-
 url_decode_q_split([$%, Hi, Lo | Tail], Ack) ->
     Hex = yaws:hex_to_integer([Hi, Lo]),
     if Hex  == 0 -> exit(badurl);
@@ -1595,6 +1582,16 @@ skip_space(T) -> T.
 %
 
 
+getvar(ARG,Key) when atom(Key) ->
+    getvar(ARG, atom_to_list(Key));
+getvar(ARG,Key) ->
+    case (ARG#arg.req)#http_request.method of
+        'POST' -> postvar(ARG, Key);
+        'GET' -> queryvar(ARG, Key);
+        _ -> undefined
+    end.
+
+
 queryvar(ARG,Key) when atom(Key) ->
     queryvar(ARG, atom_to_list(Key));
 queryvar(ARG, Key) ->
@@ -1702,8 +1699,7 @@ setconf(GC0, Groups0) ->
 	  yaws_config:can_soft_setconf(GC, Groups, OLDGC, OldGroups)} of
 	{true, true} ->
 	    yaws_config:soft_setconf(GC, Groups, OLDGC, OldGroups);
-	{true, false} when OLDGC == undefined; 
-	                   OLDGC#gconf.username == undefined ->
+	{true, false} when OLDGC == undefined -> 
 	    yaws_config:hard_setconf(GC, Groups);
 	_ ->
 	    {error, need_restart}
