@@ -1,9 +1,9 @@
 -module(yaws_cgi).
 -author('carsten@codimi.de').
 
--include_lib("yaws/include/yaws_api.hrl").
+-include("../include/yaws_api.hrl").
 -include("yaws_debug.hrl").
--include_lib("yaws/include/yaws.hrl").
+-include("../include/yaws.hrl").
 
 -export([call_cgi/5, call_cgi/4, call_cgi/3, call_cgi/2]).
 
@@ -161,10 +161,10 @@ deep_drop_prefix(_, _) ->
 
 get_socket_peername(Socket={sslsocket,_,_}) ->
     {ok, {IP, _Port}}=ssl:peername(Socket),
-    yaws:fmt_ip(IP);
+    inet_parse:ntoa(IP);
 get_socket_peername(Socket) ->
     {ok, {IP, _Port}}=inet:peername(Socket),
-    yaws:fmt_ip(IP).
+    inet_parse:ntoa(IP).
 
 
 cgi_env(Arg, Scriptfilename, Pathinfo, ExtraEnv, SC) ->
@@ -189,7 +189,7 @@ cgi_env(Arg, Scriptfilename, Pathinfo, ExtraEnv, SC) ->
 	  fun(X)->case X of {_,L} when list(L)->true;_->false end 
 	  end,
 	  ([
-	    {"SERVER_SOFTWARE", "Yaws/"++yaws_vsn:version()},
+	    {"SERVER_SOFTWARE", "Yaws/"++yaws_generated:version()},
 	    {"SERVER_NAME", Hostname},
 	    {"GATEWAY_INTERFACE", "CGI/1.1"},
 	    {"SERVER_PROTOCOL", 
@@ -235,7 +235,7 @@ tohttp_c(C) ->
 %% Get Host part from a host string that can contain host or host:port
 host(Host) ->
    case string:tokens(Host, ":") of
-       [Hostname, Port] -> Hostname;
+       [Hostname, _Port] -> Hostname;
        [Hostname]       -> Hostname;
        _Other           -> Host
    end.
