@@ -13,8 +13,8 @@
 
 -compile(export_all).
 -include_lib("kernel/include/file.hrl").
--include_lib("yaws/include/yaws.hrl").
--include_lib("yaws/include/yaws_api.hrl").
+-include("../include/yaws.hrl").
+-include("../include/yaws_api.hrl").
 -include("yaws_debug.hrl").
 
 
@@ -34,10 +34,6 @@ start(GC, true) ->
     end.
 
 
-%% syncronous start, 
-%% If we're later supposed
-%% to change uid, we're still root here.
-
 run(GC) ->
     %% First check if there is already a Yaws system running
     %% with the same sid.
@@ -82,13 +78,17 @@ run_listen(GC) ->
 			    proc_lib:init_ack(ok),
 			    aloop(L, GC);
 			error ->
-			    e("Failed to create/manipulate the ctlfile ~n"
+			    error_logger:format(
+			      "Failed to create/manipulate the ctlfile ~n"
 			      "called ~s~n"
-			      "either problems with permissions or "
+			      "Either problems with permissions or "
 			      " earlier runs of yaws ~nwith the same id "
-			      " <~p> as this, check /tmp/yaws/* for perms~n",
-			      [ctl_file(GC#gconf.id), GC#gconf.id])
-		    end;
+			      " <~p> as this, check dir for perms~n"
+			      "None of Yaws ctl functions will work~n",
+			      [ctl_file(GC#gconf.id), GC#gconf.id]),
+			    proc_lib:init_ack(ok),
+			    aloop(L, GC)
+			end;
 		Err ->
 		    e("Cannot get sockname for ctlsock: ~p",[Err] )
 	    end;
@@ -124,18 +124,19 @@ w_ctl_file(Sid, Port) ->
 
 
 ctl_file(Sid) ->
-    filename:join([yaws:tmp_dir(),
-		   "yaws",
-		   Sid,
-		   "ctl"]).
-
+    FN = filename:join([yaws_generated:ctldir(), "ctl-" ++ yaws:to_list(Sid)]),
+    filelib:ensure_dir(FN),
+    FN.
 
 
 aloop(L, GC) ->
     case gen_tcp:accept(L) of
 	{ok, A} ->
 	    handle_a(A, GC);
-	_Err ->
+	Err ->
+	    error_logger:format("yaws_ctl failed to accept: ~p~n",
+				[Err]),
+	    timer:sleep(2000),
 	    ignore
     end,
     ?MODULE:aloop(L, GC).
@@ -171,6 +172,8 @@ handle_a(A, GC) ->
 		    gen_tcp:close(A)
 	    
 	    end;
+	{error, _} ->
+	    gen_tcp:close(A);
 	_Err ->
 	    ignore
     end.
@@ -204,8 +207,6 @@ actl_trace(What) ->
 		      [What, 
 		       filename:join([GC#gconf.logdir, 
 				      "trace." ++ atom_to_list(What)])])
-	    
-	    
 	    end;
 	false ->
 	    "Need either http | traffic | off  as argument\n"
@@ -340,14 +341,14 @@ s_cmd(Fd, SID, Term) ->
 
 %% List existing yaws nodes on this machine
 ls(_) ->
-    case file:list_dir("/tmp/yaws") of
+    case file:list_dir(yaws_generated:ctldir()) of
 	{ok, List} ->
 	    io:format("~-15s~-10s~-10s~n",
 		      ["Id", "Status", "Owner"]),
 	    io:format("-------------------------------------~n",[]),
 	    lists:foreach(
-	      fun(D) ->
-		      lls(D)
+	      fun(CtlFile) ->
+		      lls(CtlFile)
 	      end, List);
 	_ ->
 	    ok
@@ -356,13 +357,16 @@ ls(_) ->
     init:stop().
 
 
-lls(Dir) ->
-    Ctl = ctl_file(Dir),
-    case {file:read_file_info(Ctl),
-	  file:read_file_info(filename:join("/tmp/yaws/", Dir))} of
-	{{ok, FI}, {ok, DI}} ->
-	    User = yaws:uid_to_name(DI#file_info.uid),
-	    Running = case connect(Dir) of
+lls(CtlFile0 = "ctl-" ++ Id) ->
+    CtlFile = filename:join([yaws_generated:ctldir(), CtlFile0]),
+    case {file:read_file_info(CtlFile),
+	  file:read_file(CtlFile)} of
+	{{ok, FI}, {error, eacces}} ->
+	    User = yaws:uid_to_name(FI#file_info.uid),
+	    io:format("~-15s~-10s~-10s~n",
+		      [Id, "noaccess", User]);
+	{{ok, FI}, {ok, _Bin}} ->
+	    Running = case connect(Id) of
 			  {ok, Sock} ->
 			      gen_tcp:close(Sock),
 			      "running";
@@ -370,30 +374,17 @@ lls(Dir) ->
 			      "hanging??";
 			  {error, eacces} ->
 			      "unknown";
-			  Err ->
+			  _Err ->
 			      "stopped"
 		      end,
+	    User = yaws:uid_to_name(FI#file_info.uid),
 	    io:format("~-15s~-10s~-10s~n",
-		      [Dir, Running, User]);
-
-
-	
-	{{ok, FI}, {error, _}} ->
-	    %% sick case,
-	    ignore;
-
-	{{error, _}, {ok, DI}} ->
-	    %% nicely terminated system
-	    User = yaws:uid_to_name(DI#file_info.uid),
-	    io:format("~-15s~-10s~-10s~n",
-		      [Dir, "stopped", User]);
-
-
-	_Err ->
-	    io:format("~-15s~-10s~-10s~n",
-		      [Dir, "unknown", "unknown"])
-		
-    end.
+		      [Id, Running, User]);
+	_ ->
+	    ok
+    end;
+lls(_) ->
+    ok.
 
 	      
 
@@ -417,7 +408,7 @@ load(X) ->
     actl(SID, {load, Modules}).
 
 check([Id, File| IncludeDirs]) ->
-    GC = yaws_config:make_default_gconf(false),
+    GC = yaws_config:make_default_gconf(false, undefined),
     GC2 = GC#gconf{include_dir = lists:map(fun(X) -> atom_to_list(X) end, 
 					   IncludeDirs),
 		   id = atom_to_list(Id)
