@@ -13,8 +13,8 @@
 
 -behaviour(gen_server).
 
--include_lib("yaws/include/yaws.hrl").
--include_lib("yaws/include/yaws_api.hrl").
+-include("../include/yaws.hrl").
+-include("../include/yaws_api.hrl").
 -include("yaws_debug.hrl").
 
 -include_lib("kernel/include/file.hrl").
@@ -63,7 +63,9 @@ gs_status() ->
 	      P ! {self(), status},
 	      receive {P, Stat} -> Stat end
       end, Pids).
-		    
+getconf() ->		    
+     gen_server:call(?MODULE,getconf).
+
 stats() -> 
     {S, Time} = status(),
     Diff = calendar:time_difference(Time, calendar:local_time()),
@@ -76,7 +78,7 @@ stats() ->
 			   E = SC#sconf.ets,
 			   L = ets:match(E, {{urlc_total, '$1'}, '$2'}),
 			   {SC#sconf.servername,  
-			    flatten(yaws:fmt_ip(SC#sconf.listen)),
+			    flatten(inet_parse:ntoa(SC#sconf.listen)),
 			    G(map(fun(P) -> list_to_tuple(P) end, L))}
 		   end, SCS)
 	 end, S#state.pairs),
@@ -95,12 +97,12 @@ l2a(A) when atom(A) -> A.
 %%          {stop, Reason}
 %%----------------------------------------------------------------------
 
-init({Debug, Trace, TraceOut, Conf, RunMod, Embedded}) ->
+init(Env) -> %% #env{Trace, TraceOut, Conf, RunMod, Embedded, Id}) ->
     process_flag(trap_exit, true),
     put(start_time, calendar:local_time()),  %% for uptime
-    case Embedded of 
+    case Env#env.embedded of 
 	false ->
-	    Config = (catch yaws_config:load(Conf, Trace, TraceOut, Debug)),
+	    Config = (catch yaws_config:load(Env)),
 	    case Config of
 		{ok, Gconf, Sconfs} ->
 		    erase(logdir),
@@ -121,7 +123,7 @@ init({Debug, Trace, TraceOut, Conf, RunMod, Embedded}) ->
 			_ ->
 			    ok
 		    end,
-		    init2(Gconf, Sconfs, RunMod, Embedded, true);
+		    init2(Gconf, Sconfs, Env#env.runmod, Env#env.embedded, true);
 		{error, E} ->
 		    case erase(logdir) of
 			undefined ->
@@ -129,7 +131,7 @@ init({Debug, Trace, TraceOut, Conf, RunMod, Embedded}) ->
 			    init:stop(),
 			    {stop, E};
 			Dir ->
-			    GC = yaws_config:make_default_gconf(true),
+			    GC = yaws_config:make_default_gconf(true, Env#env.id),
 			    yaws_log:setdir(GC#gconf{logdir = Dir}, []),
 			    error_logger:error_msg("Yaws: bad conf: ~s "
 						   "terminating~n",[E]),
@@ -142,7 +144,7 @@ init({Debug, Trace, TraceOut, Conf, RunMod, Embedded}) ->
 	    end;
 	true ->
 	    {ok, #state{gc = undefined,
-			embedded = Embedded,
+			embedded = Env#env.embedded,
 			pairs = [],
 			mnum = 0}}
     end.
@@ -174,25 +176,7 @@ init2(GC, Sconfs, RunMod, Embedded, FirstTime) ->
     L2 = lists:zf(
 	   fun(Group) -> start_group(GC, Group) end,
 	   Sconfs),
-
-    yaws_log:uid_change(GC),
-    
-
-    %% and now finally, we've opened the ctl socket and are
-    %% listening to all sockets we can possibly change uid
-
-    GC2 = case (catch yaws:setuser(GC#gconf.username)) of
-	      ignore ->
-		  GC;
-	      {ok, NewUid} ->
-		  error_logger:info_msg("Changed uid to ~s~n", [NewUid]),
-		  GC#gconf{uid = NewUid};
-	      Other ->
-		  error_logger:error_msg("Failed to set user:~n~p", [Other]),
-		  erlang:fault(Other)
-	  end,
-    foreach(fun({Pid, _}) -> Pid ! {newuid, GC2#gconf.uid} end, L2),
-    {ok, #state{gc = GC2,
+    {ok, #state{gc = GC,
 		pairs = L2,
 		mnum = 0,
 		embedded = Embedded}}.
@@ -277,9 +261,14 @@ handle_call({update_sconf, NewSc}, From, State) ->
 					  [yaws:sconf_to_srvstr(OldSc)]),
 		    {reply, ok, State};
 		false ->
-		    Pid ! {update_sconf, NewSc, OldSc, From},
-		    P2 = yaws_config:update_sconf(NewSc, State#state.pairs),
-		    {noreply, State#state{pairs = P2}}
+		    Pid ! {update_sconf, NewSc, OldSc, From, self()},
+		    receive
+			{updated_sconf, NewSc2} ->			
+			    P2 = yaws_config:update_sconf(NewSc2, State#state.pairs),
+			    {noreply, State#state{pairs = P2}}
+		    after 1000 ->
+			    {reply, {error, "Failed to update new conf"}, State}
+		    end
 	    end;
 	false ->
 	    {reply, {error, "No matching group"}, State}
@@ -318,8 +307,8 @@ handle_call({add_sconf, SC}, From, State) ->
 		false ->
 		    {reply, ok, State};
 		{true, Pair} ->
-		    Pid = element(1, Pair),
-		    Pid ! {newuid, GC#gconf.uid},
+		    %Pid = element(1, Pair),
+		    %Pid ! {newuid, GC#gconf.uid},
 		    P2 = [Pair | State#state.pairs],
 		    {reply, ok, State#state{pairs = P2}}
 	    end
@@ -378,8 +367,18 @@ handle_info(_Msg, State) ->
 terminate(_Reason, _State) ->
     ok.
 
-do_listen(SC) ->
+do_listen(GC, SC) ->
     case SC#sconf.ssl of
+	undefined when ?gc_use_fdsrv(GC) ->
+	    %% Use fdsrv kit from jungerl, requires fdsrv
+	    %% to be properly installed
+	    fdsrv:start(),
+	    case fdsrv:bind_socket(tcp, SC#sconf.port) of
+		{ok, Fd} ->
+		    {nossl, gen_tcp_listen(SC#sconf.port,[{fd, Fd}|opts(SC)])};
+		Err ->
+		    {nossl, Err}
+	    end;
 	undefined ->
 	    {nossl, gen_tcp_listen(SC#sconf.port, opts(SC))};
 	SSL ->
@@ -391,18 +390,6 @@ gen_tcp_listen(Port, Opts) ->
     gen_tcp:listen(Port, Opts).
 
 
-set_writeable(Dir) ->
-    set_dir_mode(Dir, 8#777).
-
-set_dir_mode(Dir, Mode) ->
-    case file:read_file_info(Dir) of
-	{ok, FI} ->
-	    file:write_file_info(Dir, FI#file_info{mode = Mode});
-	_Err ->
-	    error_logger:format("Failed to read_file_info(~s)~n", [Dir])
-    end.
-
-
 gserv(_Top, _, []) ->
     proc_lib:init_ack(none);
 
@@ -415,12 +402,12 @@ gserv(Top, GC, Group0) ->
     Group = map(fun(SC) -> setup_ets(SC)
 		end, Group0),
     SC = hd(Group),
-    case do_listen(SC) of
+    case do_listen(GC, SC) of
 	{SSLBOOL, {ok, Listen}} ->
 	    call_start_mod(SC),
 	    error_logger:info_msg(
 	      "Yaws: Listening to ~s:~w for servers~s~n",
-	      [yaws:fmt_ip(SC#sconf.listen),
+	      [inet_parse:ntoa(SC#sconf.listen),
 	       SC#sconf.port,
 	       catch map(
 		       fun(S) ->  
@@ -430,11 +417,7 @@ gserv(Top, GC, Group0) ->
 		       end, Group)
 	      ]),
 	    proc_lib:init_ack({self(), Group}),
-	    N = receive
-		    {newuid, UID} ->
-			UID
-		end,
-	    GS = #gs{gconf = GC#gconf{uid = N},
+	    GS = #gs{gconf = GC,
 		     group = Group,
 		     ssl = SSLBOOL,
 		     l = Listen},
@@ -442,7 +425,7 @@ gserv(Top, GC, Group0) ->
 	    gserv_loop(GS, [], 0, Last);
 	{_,Err} ->
 	    error_logger:format("Yaws: Failed to listen ~s:~w  : ~p~n",
-				[yaws:fmt_ip(SC#sconf.listen),
+				[inet_parse:ntoa(SC#sconf.listen),
 				 SC#sconf.port, Err]),
 	    proc_lib:init_ack({error, "Can't listen to socket: ~p ",[Err]}),
 	    exit(normal)
@@ -450,15 +433,10 @@ gserv(Top, GC, Group0) ->
 			
 
 setup_dirs(GC) ->
-    TD0 = filename:join([yaws:tmp_dir(),"yaws"]),
+    TD0 = filename:join([GC#gconf.tmpdir,"yaws"]),
     file:make_dir(TD0),
-    set_dir_mode(TD0, 8#777),
-
     TD1 = filename:join([TD0, GC#gconf.id]),
     file:make_dir(TD1),
-    set_dir_mode(TD1, 8#755),
-    yaws:uid_change_files(GC, TD1, []),
-
     case file:list_dir(TD1) of
 	{ok, LL} ->
 	    foreach(
@@ -479,6 +457,18 @@ setup_ets(SC) ->
     ets:insert(E, {num_bytes, 0}),
     SC#sconf{ets = E}.
 
+clear_ets_complete(SC) ->
+    case SC#sconf.ets of
+	undefined ->
+ 	    setup_ets(SC);
+ 	E ->
+ 	    ets:match_delete(E,'_'),
+ 	    ets:insert(E, {num_files, 0}),
+ 	    ets:insert(E, {num_bytes, 0}),
+ 	    SC
+     end.
+
+
 gserv_loop(GS, Ready, Rnum, Last) ->
     receive
 	{From , status} ->
@@ -488,7 +478,7 @@ gserv_loop(GS, Ready, Rnum, Last) ->
 	    New = acceptor(GS),
 	    gserv_loop(GS, Ready, Rnum, New);
 	{_From, next} ->
-	    [R|RS] = Ready,
+	    [{_Then, R}|RS] = Ready,
 	    R ! {self(), accept},
 	    gserv_loop(GS, RS, Rnum-1, R);
 	{From, done_client, Int} ->
@@ -502,7 +492,8 @@ gserv_loop(GS, Ready, Rnum, Last) ->
 		    From ! {self(), stop},
 		    gserv_loop(GS2, Ready, Rnum, Last);
 		Rnum < 8 ->
-		    gserv_loop(GS2, [From | Ready], Rnum+1, Last)
+		    %% cache this process for 10 secs
+		    gserv_loop(GS2, [{now(), From} | Ready], Rnum+1, Last)
 	    end;
 	{'EXIT', Pid, Reason} ->
 	    case get(top) of
@@ -528,7 +519,7 @@ gserv_loop(GS, Ready, Rnum, Last) ->
 
 	%% This code will shutdown all ready procs as well as the
 	%% acceptor() 
-	{update_sconf, NewSc, OldSc, From} ->
+	{update_sconf, NewSc, OldSc, From, Updater} ->
 	    case lists:member(OldSc, GS#gs.group) of
 		false ->
 		    error_logger:error_msg("gserv: No found SC ~p/~p~n",
@@ -536,13 +527,14 @@ gserv_loop(GS, Ready, Rnum, Last) ->
 		    erlang:fault(nosc);
 		true ->
 		    stop_ready(Ready, Last),
-		    ets:delete(OldSc#sconf.ets),
-		    GS2 = GS#gs{group = [setup_ets(NewSc) | 
+		    NewSc2 = clear_ets_complete(NewSc),
+		    GS2 = GS#gs{group = [ NewSc2 | 
 					 lists:delete(OldSc,GS#gs.group)]},
 		    Ready2 = [],
+		    Updater ! {updated_sconf, NewSc2},
 		    gen_server:reply(From, ok),
 		    error_logger:info_msg("Updating sconf for server ~s~n",
-					  [yaws:sconf_to_srvstr(NewSc)]),
+					  [yaws:sconf_to_srvstr(NewSc2)]),
 		    New = acceptor(GS2),
 		    gserv_loop(GS2, Ready2, 0, New)
 	    end;
@@ -574,7 +566,7 @@ gserv_loop(GS, Ready, Rnum, Last) ->
 				  [yaws:sconf_to_srvstr(SC)]),
 	    New = acceptor(GS2),
 	    gserv_loop(GS2, Ready2, 0, New);
-
+ 
 	{update_gconf, GC} ->
 	    stop_ready(Ready, Last),
 	    GS2 = GS#gs{gconf = GC},
@@ -583,16 +575,29 @@ gserv_loop(GS, Ready, Rnum, Last) ->
 	    error_logger:info_msg("Updating gconf \n",[]),
 	    New = acceptor(GS2),
 	    gserv_loop(GS2, Ready2, 0, New)
-
+    after (10 * 1000) ->
+	    %% collect old procs, to save memory
+	    {NowMega, NowSecs, _} = now(),
+	    R2 = lists:filter(fun({{ThenMega, ThenSecs, _}, Pid}) ->
+				      if
+					  NowMega > ThenMega;
+					  (NowSecs > (ThenSecs + 8)) ->
+					      Pid ! {self(), stop},
+					      false;
+					  true ->
+					      true
+				      end
+			      end, Ready),
+	    gserv_loop(GS, R2, Rnum, Last)
     end.
-	    
+
 
 stop_ready(Ready, Last) ->
     unlink(Last),
     exit(Last, shutdown),
     
     lists:foreach(
-      fun(Pid) -> 
+      fun({_,Pid}) -> 
 	      Pid ! {self(), stop}
       end, Ready).
 
@@ -690,8 +695,6 @@ ssl_opts(SSL) ->
 filter_false(L) ->
     [X || X <- L, X /= false].
 
-	   
-	 
 do_accept(GS) when GS#gs.ssl == nossl ->
     ?Debug("wait in accept ... ~n",[]),
     gen_tcp:accept(GS#gs.l);
@@ -699,10 +702,22 @@ do_accept(GS) when GS#gs.ssl == ssl ->
     ssl:accept(GS#gs.l,10000).
 
 
-initial_acceptor(GS=#gs{ssl=nossl})->
-    acceptor(GS);
-initial_acceptor(GS=#gs{ssl=ssl}) ->
-    initial_acceptor(GS, 5).
+
+initial_acceptor(GS) ->
+    if
+	GS#gs.ssl == nossl ->
+	    acceptor(GS);
+	GS#gs.ssl == ssl ->
+	    initial_acceptor(GS, 5)
+    end.
+
+
+% an SSL connection might include user interaction.
+% So while one person has a dialog box on the screen asking him to review
+% the server's certificate, no other person is able to establish a new
+% connnection to the server.
+% unless we run multiple SSL acceptor processes
+% Therefore, the code below!
 
 initial_acceptor(_GS, 0) ->
     ok;
@@ -738,7 +753,7 @@ acceptor0(GS, Top) ->
 					       inet:peername(Client)
 				       end,
 		    Str = ?F("New (~p) connection from ~s:~w~n", 
-			[GS#gs.ssl, yaws:fmt_ip(IP),Port]),
+			[GS#gs.ssl, inet_parse:ntoa(IP),Port]),
 		    yaws_log:trace_traffic(from_client, Str);
 		_ ->
 		    ok
@@ -770,6 +785,7 @@ acceptor0(GS, Top) ->
 		{Top, stop} ->
 		    exit(normal);
 		{Top, accept} ->
+		    erase_transients(),
 		    acceptor0(GS, Top)
 	    end;
 	{error, timeout} ->
@@ -832,7 +848,11 @@ aloop(CliSock, GS, Num) ->
 erase_transients() ->
     I = get(init_db),
     erase(),
-    lists:foreach(fun({K,V}) -> put(K,V) end, I).
+    if I == undefined ->
+	    ok;
+       list(I) ->
+	    lists:foreach(fun({K,V}) -> put(K,V) end, I)
+    end.
 
 
 handle_method_result(Res, CliSock, IP, GS, Req, H, Num) ->
@@ -908,24 +928,55 @@ fix_abs_uri(Req, H) ->
 	_ -> {Req, H}
     end.
 
-pick_sconf(_GS, _H, Group, ssl) ->
-    hd(Group);
 
-pick_sconf(GS, H, Group, nossl) ->
+%% compare servername and ignore any optional
+%% :Port postfix
+comp_sname([$:|_], _) -> true;
+comp_sname(_, [$:|_]) -> true;
+comp_sname([H|T], [H|T1]) -> comp_sname(T,T1);
+comp_sname([],[])-> true;
+comp_sname(_,_)-> false.
+   
+    
+
+
+pick_sconf(GC, H, [SC|_Group], ssl) ->
+    case comp_sname(H#headers.host, SC#sconf.servername) of
+	true -> 
+	    SC;
+	false when ?gc_pick_first_virthost_on_nomatch(GC)  ->
+	    SC;
+	false ->
+	    yaws_debug:format("Drop req since ~p =/ ~p \n", 
+			      [H#headers.host, SC#sconf.servername]),
+	    exit(normal)
+    end;
+
+
+pick_sconf(GC, H, Group, nossl) ->
     case H#headers.host of
-	undefined ->
+	undefined when ?gc_pick_first_virthost_on_nomatch(GC) ->
 	    hd(Group);
 	Host ->
-	    pick_host(GS, Host, Group, Group)
+	    pick_host(GC, Host, Group, Group)
+    end.
+
+
+pick_host(GC, Host, [SC|T], Group) ->
+    case comp_sname(Host, SC#sconf.servername) of
+	true -> SC;
+	false -> pick_host(GC, Host, T, Group)
+    end;
+pick_host(GC, Host, [], Group) ->
+    if ?gc_pick_first_virthost_on_nomatch(GC) ->
+	    hd(Group);
+       true ->
+	    yaws_debug:format("Drop req since ~p doesn't match any servername \n", 
+			      [Host]),
+	    exit(normal)
     end.
 
 
-pick_host(_GC, Host, [H|_T], _Group) when H#sconf.servername == Host ->
-    H;
-pick_host(GC, Host, [_|T], Group) ->
-    pick_host(GC, Host, T, Group);
-pick_host(_GC, _Host, [], Group) ->
-    hd(Group).
 
 
 inet_peername(Sock, SC) ->
@@ -1046,14 +1097,14 @@ bad_request(CliSock, Req, _Head) ->
 	       port(CliSock) ->
 		   case inet:peername(CliSock) of
 		       {ok, {IP, _Port}} ->
-			   yaws:fmt_ip(IP);
+			   inet_parse:ntoa(IP);
 		       _ ->
 			   "unknown"
 		   end;
 	       true ->
 		   case ssl:peername(CliSock) of
 		       {ok, {IP, _Port}} ->
-			   yaws:fmt_ip(IP);
+			   inet_parse:ntoa(IP);
 		       _ ->
 			   "unknown"
 		   end
@@ -1162,7 +1213,19 @@ not_implemented(CliSock, Req, Head) ->
     deliver_options(CliSock, Req).
 
 'PUT'(CliSock, Req, Head) ->
-    body_method(CliSock, Req, Head).
+    ?Debug("PUT Req=~p~n H=~p~n", [?format_record(Req, http_request),
+				   ?format_record(Head, headers)]),
+    SC=get(sc),
+    ok = yaws:setopts(CliSock, [{packet, raw}, binary], is_ssl(SC#sconf.ssl)),
+    PPS = SC#sconf.partial_post_size,
+    CT = 
+	case yaws:lowercase(Head#headers.content_type) of
+	    "multipart/form-data"++_ -> multipart;
+	    _ -> urlencoded
+	end,
+    ARG = make_arg(CliSock, Head, Req, undefined),
+    handle_request(CliSock, ARG, p).
+    
 
 'DELETE'(CliSock, Req, Head) ->
     no_body_method(CliSock, Req, Head).
@@ -1177,6 +1240,12 @@ not_implemented(CliSock, Req, Head) ->
     %%                    ?format_record(Head, headers)]),
     body_method(CliSock, Req, Head).
 
+'MOVE'(CliSock, Req, Head) ->
+    no_body_method(CliSock, Req, Head).
+
+'COPY'(CliSock, Req, Head) ->
+    no_body_method(CliSock, Req, Head).
+
 
 body_method(CliSock, Req, Head) ->
     SC=get(sc),
@@ -1244,6 +1313,10 @@ handle_extension_method("PROPFIND", CliSock, Req, Head) ->
     'PROPFIND'(CliSock, Req, Head);   
 handle_extension_method("MKCOL", CliSock, Req, Head) ->                      
     'MKCOL'(CliSock, Req, Head);                                             
+handle_extension_method("MOVE", CliSock, Req, Head) ->                      
+    'MOVE'(CliSock, Req, Head);                                             
+handle_extension_method("COPY", CliSock, Req, Head) ->                      
+    'COPY'(CliSock, Req, Head);                                             
 handle_extension_method(_Method, CliSock, Req, Head) ->
     not_implemented(CliSock, Req, Head).
 
@@ -1263,15 +1336,15 @@ handle_request(CliSock, ARG, N) ->
 		    SC=get(sc),
 		    ?Debug("Test revproxy: ~p and ~p~n", 
 			[DecPath, SC#sconf.revproxy]),
-
-		    case {is_auth(ARG, DecPath,
-				  ARG#arg.headers,SC#sconf.authdirs),
+		    IsAuth = is_auth(ARG, DecPath,ARG#arg.headers,
+				     SC#sconf.authdirs),
+		    case {IsAuth, 
 			  is_revproxy(DecPath, SC#sconf.revproxy),
 			  is_redirect_map(DecPath, SC#sconf.redirect_map)} of
 			{_, _, {true, MethodHostPort}} ->
 			    deliver_302_map(CliSock, Req, ARG, MethodHostPort);
 			{true, false, _} ->
-			    UT   = url_type(DecPath),
+			    UT   = url_type(DecPath, ARG#arg.docroot),
 			    ARG2 = ARG#arg{server_path = DecPath,
 					   querydata= QueryPart,
 					   fullpath=UT#urltype.fullpath,
@@ -1295,7 +1368,8 @@ handle_request(CliSock, ARG, N) ->
 
 is_auth(_ARG, _Req_dir, _H, [] ) -> 
     true;
-is_auth(ARG, Req_dir,H,[{Auth_dir, #auth{realm=Realm, users=Users}}|T] ) ->
+is_auth(ARG, Req_dir,H,[{Auth_dir, 
+			 #auth{realm=Realm, users=Users, pam=Pam}}|T] ) ->
     case lists:prefix(Auth_dir, Req_dir) of
 	true ->
 	    case H#headers.authorization of
@@ -1307,9 +1381,18 @@ is_auth(ARG, Req_dir,H,[{Auth_dir, #auth{realm=Realm, users=Users}}|T] ) ->
 			true ->
 			    maybe_auth_log({ok, User}, ARG),
 			    true;
-			false ->
+			false when Pam == false ->
 			    maybe_auth_log({401, User, Password}, ARG),
-			    {false, Realm}
+			    {false, Realm};
+			false when Pam /= false ->
+			    case yaws_pam:auth(User, Password) of
+				{yes, _} ->
+				    maybe_auth_log({ok, User}, ARG),
+				    true;
+				{no, _Rsn} ->
+				    maybe_auth_log({401, User, Password}, ARG),
+				    {false, Realm}
+			    end
 		    end
 	    end;
 	false ->
@@ -1380,8 +1463,9 @@ handle_ut(CliSock, ARG, UT, N) ->
 	regular ->
 	    ETag = yaws:make_etag(UT#urltype.finfo),
 	    Range = case H#headers.if_range of
-			Range_etag = [$"|_] when Range_etag /= ETag ->
-			    all;
+%			[$"|_] = Range_etag when Range_etag /= ETag -> 
+			[34|_] = Range_etag when Range_etag /= ETag -> 
+			    all; 
 			_ ->
 			    requested_range(H#headers.range, 
 					    (UT#urltype.finfo)#file_info.size)
@@ -1469,11 +1553,51 @@ handle_ut(CliSock, ARG, UT, N) ->
 			     end,
 			     fun(A)->finish_up_dyn_file(A, CliSock)
 			     end
-			    )
+			    );
+	dav ->
+	    Next =
+		if
+		    Req#http_request.method == 'PUT' ->
+			fun(A) -> yaws_dav:put(SC, A) end;
+		    Req#http_request.method == 'DELETE' ->
+			fun(A) -> yaws_dav:delete(A) end;
+		    Req#http_request.method == "PROPFIND" ->
+			fun(A)-> yaws_dav:propfind(A) end;
+		    Req#http_request.method == "MOVE" ->
+			fun(A)-> yaws_dav:move(A) end;
+		    Req#http_request.method == "COPY" ->
+			fun(A)-> yaws_dav:copy(A) end;
+		    Req#http_request.method == "MKCOL" ->
+			fun(A)-> yaws_dav:mkcol(A) end;
+		    Req#http_request.method == 'GET';
+		    Req#http_request.method == 'HEAD' ->
+			case prim_file:read_file_info(UT#urltype.fullpath) of
+			    {ok, FI} when FI#file_info.type == regular ->
+				{regular, FI};
+			    _ ->
+				error
+			end;
+		    true ->
+			error
+		end,
+	    case Next of
+		error ->
+		    handle_ut(CliSock, ARG, #urltype{type = error}, N);
+		{regular, Finfo} ->
+		    handle_ut(CliSock, ARG, UT#urltype{type = regular,
+						       finfo = Finfo}, N);
+		_ ->
+		    yaws:outh_set_dyn_headers(Req, H, UT),
+		    deliver_dyn_part(CliSock, 
+				     0, "dav",
+				     N,
+				     ARG,UT,
+				     Next,
+				     fun(A)->finish_up_dyn_file(A, CliSock)  end
+				    )
+	    end
     end.
 
-
-
 done_or_continue() ->
     case yaws:outh_get_doclose() of
 	true -> done;
@@ -1644,7 +1768,7 @@ do_yaws(CliSock, ARG, UT, N) ->
 					      Es == 0 ->
 	    deliver_dyn_file(CliSock, Spec, ARG, UT, N);
 	Other  ->
-	    del_old_files(Other),
+	    del_old_files(get(gc),Other),
 	    {ok, [{errors, Errs}| Spec]} = 
 		yaws_compile:compile_file(UT#urltype.fullpath),
 	    ?Debug("Spec for file ~s is:~n~p~n",[UT#urltype.fullpath, Spec]),
@@ -1653,12 +1777,12 @@ do_yaws(CliSock, ARG, UT, N) ->
     end.
 
 
-del_old_files([]) ->
+del_old_files(_, []) ->
     ok;
-del_old_files([{_FileAtom, spec, _Mtime1, Spec, _}]) ->
+del_old_files(GC, [{_FileAtom, spec, _Mtime1, Spec, _}]) ->
     foreach(
       fun({mod, _, _, _,  Mod, _Func}) ->
-	      F=filename:join([yaws:tmp_dir(), "yaws", 
+	      F=filename:join([GC#gconf.tmpdir, "yaws", 
 			       yaws:to_list(Mod) ++ ".erl"]),
 	      code:purge(Mod),
 	      code:purge(Mod),
@@ -1713,9 +1837,6 @@ get_client_data_all(CliSock, Bs, SSlBool) ->
 	    exit(normal)
     end.
 
-
-
-
 %% Return values:
 %% continue, done, {page, Page}
 
@@ -1952,8 +2073,6 @@ end_streaming({Z, Priv}, CliSock) ->
     done_or_continue().
 
     
-
-
 %% what about trailers ??
 
 skip_data(List, Fd, Sz) when list(List) ->
@@ -2028,7 +2147,7 @@ handle_out_reply(L, LineNo, YawsFile, UT, A) when list (L) ->
 %% yssi, yaws include
 handle_out_reply({yssi, Yfile}, LineNo, YawsFile, UT, [ARG]) ->
     SC = get(sc),
-    UT2 = url_type(lists:flatten(UT#urltype.dir) ++ [$/|Yfile]),
+    UT2 = url_type(lists:flatten(UT#urltype.dir) ++ [$/|Yfile], ARG#arg.docroot),
     case UT2#urltype.type of
 	yaws ->
 	    Mtime = mtime(UT2#urltype.finfo),
@@ -2040,7 +2159,7 @@ handle_out_reply({yssi, Yfile}, LineNo, YawsFile, UT, [ARG]) ->
 						      Es == 0 ->
 		    deliver_dyn_file(CliSock, Spec ++ [yssi], ARG, UT2, N);
 		Other  ->
-		    del_old_files(Other),
+		    del_old_files(get(gc), Other),
 		    {ok, [{errors, Errs}| Spec]} = 
 			yaws_compile:compile_file(UT2#urltype.fullpath),
 		    ?Debug("Spec for file ~s is:~n~p~n",
@@ -2500,20 +2619,16 @@ get_more_post_data(PPS, ARG) ->
 	    case get_client_data(ARG#arg.clisock, N, 
 			       is_ssl(SC#sconf.ssl)) of
 		Bin when binary(Bin) ->
-%		    io:format("Got ~p\n", [size(Bin)]),
 		    {partial, Bin};
 		Else ->
-%		    io:format("Got error ~p\n", [Else]),
 		    {error, Else}
 	    end;
        true ->
 	    case get_client_data(ARG#arg.clisock, Len - PPS, 
 			       is_ssl(SC#sconf.ssl)) of
 		Bin when binary(Bin) ->
-%		    io:format("Got tail ~p\n", [size(Bin)]),
 		    Bin;
 		Else ->
-%		    io:format("Got tail error ~p\n", [Else]),
 		    {error, Else}
 	    end
     end.
@@ -2719,14 +2834,14 @@ now_secs() ->
 
 
 %% a file cache,
-url_type(GetPath) ->
+url_type(GetPath, ArgDocroot) ->
     SC=get(sc),
     GC=get(gc),
     E = SC#sconf.ets,
     update_total(E, GetPath),
     case ets:lookup(E, {url, GetPath}) of
 	[] ->
-	    UT = do_url_type(SC, GetPath),
+	    UT = do_url_type(SC, GetPath, ArgDocroot),
 	    ?TC([{record, UT, urltype}]),
 	    ?Debug("UT=~s\n", [?format_record(UT, urltype)]),
 	    CF = cache_file(SC, GC, GetPath, UT),
@@ -2740,7 +2855,7 @@ url_type(GetPath) ->
 		    ?Debug("Timed out entry for ~s ~p~n", 
 			[GetPath, {When, N}]),
 		    %% more than 30 secs old entry
-		    UT2 = do_url_type(SC, GetPath),
+		    UT2 = do_url_type(SC, GetPath, ArgDocroot),
 		    case file_changed(UT, UT2) of
 			true ->
 			    ?Debug("Recaching~n", []),
@@ -2828,7 +2943,7 @@ cache_file(SC, GC, Path, UT) when
 			case ?sc_has_deflate(SC)
 			    and (UT#urltype.type==regular) of
 			    true ->
-				case zlib:gzip(Bin) of
+				case yaws_zlib:gzip(Bin) of
 				    {ok, DB} when binary(DB), 
 						  size(DB)*10<size(Bin)*9 ->
 					?Debug("storing deflated version "
@@ -2878,12 +2993,23 @@ clear_ets(E) ->
     
 
 %% return #urltype{}
-do_url_type(SC, GetPath) ->
+do_url_type(SC, GetPath, ArgDocroot) ->
     ?Debug("do_url_type SC=~s~nGetPath=~p~n", 
 	   [?format_record(SC,sconf), GetPath]),
     case GetPath of
+	_ when ?sc_has_dav(SC) ->
+	    {Comps, RevFile} = split(GetPath, [], []),
+	    {_Type, Mime} = suffix_type(RevFile),
+	    DR = ArgDocroot,
+	    FullPath = [DR, GetPath],
+	    #urltype{type = dav, 
+		     dir = Comps,
+		     getpath = GetPath,
+		     path = GetPath,
+		     fullpath = FullPath,
+		     mime = Mime};
 	"/" -> %% special case 
-	    maybe_return_dir(SC#sconf.docroot, GetPath);
+	    maybe_return_dir(ArgDocroot, GetPath);
 	[$/, $~ |Tail] ->
 	    ret_user_dir(Tail);
 	_ ->
@@ -2891,7 +3017,7 @@ do_url_type(SC, GetPath) ->
 	    ?Debug("Comps = ~p RevFile = ~p~n",[Comps, RevFile]),
 	    case check_appmods(SC#sconf.appmods, Comps, RevFile) of
 		false ->
-		    DR = SC#sconf.docroot,
+		    DR = ArgDocroot,
 		    FullPath = [DR, GetPath],
 		    ?Debug("FullPath = ~p~n", [FullPath]),
 		    case prim_file:read_file_info(FullPath) of
@@ -3007,11 +3133,11 @@ maybe_return_dir(DR, GetPath) ->
     ?Debug("maybe_return_dir(~p, ~p)", [DR, GetPath]), 
     case prim_file:read_file_info([DR, GetPath, "index.yaws"]) of
 	{ok, FI} when FI#file_info.type == regular ->
-	    do_url_type(get(sc), GetPath ++ "index.yaws");
+	    do_url_type(get(sc), GetPath ++ "index.yaws", DR);
 	_ ->
 	    case prim_file:read_file_info([DR, GetPath, "index.html"]) of
 		{ok, FI} when FI#file_info.type == regular ->
-		    do_url_type(get(sc), GetPath ++ "index.html");
+		    do_url_type(get(sc), GetPath ++ "index.html", DR);
 		_ ->
 		    case file:list_dir([DR, GetPath]) of
 			{ok, List} ->
@@ -3154,7 +3280,7 @@ ret_user_dir(Upath)  ->
 				    allowed_scripts = [],
 				    docroot=DR2},
 			    put(sc, SC2),
-			    redir_user(do_url_type(SC2, Path), User)%% recurse
+			    redir_user(do_url_type(SC2, Path, DR2), User)%% recurse
 		    end;
 		{redir_dir, User} ->
 		    #urltype {type = redir,
@@ -3263,6 +3389,8 @@ ssl_flush(_Sock, undefined) ->
     ok;
 ssl_flush(_Sock, 0) ->
     ok;
+ssl_flush(Sock, Sz) when list(Sz) ->
+    ssl_flush(Sock, strip_list_to_integer(Sz));
 ssl_flush(Sock, Sz) ->
     yaws:split_recv(ssl:recv(Sock, Sz, 1000), Sz).
 
